# ssb-hyper-blobs

A scuttlebutt plugin which runs an `artefact-server` instance, providing hyper/DAT based blob storage.


## Example Usage

```
const caps = require('ssb-caps')
const config = require('ssb-config')

const stack = require('secret-stack')({ caps })
  .use(require('ssb-db'))
  .use(require('ssb-hyper-blobs'))

const ssb = stack(config)

ssb.hyperBlobs.add
```

## Behaviour

On plugin init, starts two servers:
1. An http server for serving blobs
2. A hyper server which handles p2p replication of blobs


When you request a particular blob from the http server, if you already have it locally it will be immediately served.
Otherwise, it will be replicated from remote peers (where possible), then served.

When you connect to another scuttlebutt peer, if they also have `ssb-hyper-blobs` installed, you will make an RPC call on them,
registering your hyper driveAddress with them.

Further, all scuttlebutt messages are scanned for references to driveAddresses, and these are registered.

Stores files in `path.join(config.path, hyperBlobs)`

### Pataka mode

A pataka is an always-online peer, with the following differences: 
- it does full replication of drives (in contrast, other peers do _sparse replication_ - only fetch files they want to view)
- it has no encryption keys for files, so it cannot read anything it has stored / replicated
- it does not have methods for adding files to its own drive


## Config

This module can be configured by modifying config passed into the secret-stack startup.
Here's the default config values if nothing is passed in:

```js
{
  hyperBlobs: {
    port: 26836,
    pataka: false
  }
}
```

- `port` *Number* - the port the http server will start on
- `pataka` *Boolean* - whether the ssb instance is running in Pataka mode (see Behaviour above)

## API

### `ssb.hyperBlobs.add(cb) => sink`

Creates a pull-stream sink that works well with e.g. `pull-files`
- `cb` *Function* - a callback which is run which reveives `err, data`, where data is the info needed to read the blob:
    - `driveAddress` - the address of the hyper drive
    - `blobId`
        - an id unique to this blob (within this particular drive)
        - NOT a hash of the content (currently generated by [uuid](npmjs.com/uuid)
    - `readKey`
        - the encryption key you need if you want to read the contents of the blob

### `ssb.hyperBlobs.registerDrive(driveAddress, cb)`

This method is mainly for use as a remote call by a peer that has connected to you, 
allowing them to register their drives with you.


### `GET http://localhost:PORT/drive/:driveAddress/blob/:blobId?readKey=READ_KEY&start=START&end=END&fileName=FILENAME`

params:
- `driveAddress` - address of drive where blob is stored (hex)
- `blobId` - id of blob within that store

query:
- `readKey` *String* (hex)
    - decryption key which allows you to read the content of the blob 
    - **required** (may not be required in future to allow pulling the encrypted blob)
- `start` *Number* (optional)
    - byte offset to begin stream
    - default: `0`
- `end` *Number* (optional)
    - byte offset to stop stream
    - default: `EOF`
- `fileName` *String* (optional)
    - if provided, allows the response to encode a mimeType
    - currently only the extension of the fileName is used


PORT is the whatever is configured as `hyperBlobs.port`

## TODO

- [ ] bound how drives are auto-registered?
    - [ ] only accept RPC registration of drives which are from people you follow / are in a group with
        - this would stop patakas being abused by people connecting who are not "members" of that pataka
    - [ ] only auto-register drives from friends messages?

