const ArtefactServer = require('artefact-server')
const { artefactEncryptionKey } = require('artefact-store')
const path = require('path')
const Obz = require('obz')
const defer = require('pull-defer')
const toPull = require('stream-to-pull-stream')
const { v4: uuid } = require('uuid') // TODO check which version to use!

module.exports = {
  name: 'hyperBlobs',
  version: require('./package.json').version,
  manifest: {
    add: 'sink',
    registerDrive: 'async'
  },
  permissions: {
    anonymous: {
      allow: ['registerDrive']
    }
  },
  init (ssb, config) {
    const {
      port = 26836,
      // pataka = false
    } = (config.hyperBlobs || {})

    const server = Obz() // artefact-store
    ArtefactServer({
      storage: path.resolve(config.path, 'hyperBlobs'),
      port
    })
      .then(_server => server.set(_server))
      .catch(err => {
        throw err // TODO something?
      })

    ssb.close.hook(function (close, args) {
      const _this = this
      console.time('hyperBlobs close')
      server.once(server => {
        server.close()
          .then(() => {
            console.timeEnd('hyperBlobs close') // 8 seconds!
            close.apply(_this, args)
          })
      })
    })

    ssb.on('rpc:connect', remote => {
      if (remote.id === ssb.id) return // could be ssb-client connection

      if (!remote.hyperBlobs) return
    })

    return {
      add (cb) {
        const sink = defer.sink()

        const blobId = uuid()
        const readKey = artefactEncryptionKey()
        const opts = {}

        server.once(server => {
          const writeStream = server.store.createWriteStream(blobId, readKey, opts)

          const _sink = toPull.sink(writeStream, (err) => {
            if (err) return cb(err)

            // TODO - check this is needed?
            server.store.fileReady(blobId)
              .then(() => {
                cb(null, {
                  driveAddress: server.store.driveAddress.toString('hex'),
                  blobId,
                  readKey: readKey.toString('hex')
                  // fileName....
                })
              })
          })
          sink.resolve(_sink)
        })

        return sink
      },
      registerDrive () {}
    }
  }
}
